import asyncio
import struct


async def handle_echo(reader, writer):
    data = struct.pack(">H", 5)
    writer.write(data)
    print("Sending")
    await writer.drain()


async def main():
    server = await asyncio.start_server(handle_echo, "0.0.0.0", 8888)

    addr = server.sockets[0].getsockname()
    print(f"Serving on {addr}")

    async with server:
        await server.serve_forever()


asyncio.run(main())



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
===== 
Benchmark PCAP Creaion
===== 

Benchmark the creation and parsing of a biug pcap file
"""
__author__ = "Diarmuid Collins"
__copyright__ = "Copyright 2018"
__version__ = "0.0.1"
__maintainer__ = "Diarmuid Collins"
__email__ = "dcollins@curtisswright.com"
__status__ = "Production"


import sys
sys.path.append("..")

import time
import struct
import AcraNetwork.IENA as iena
import AcraNetwork.iNetX as inetx
import AcraNetwork.Pcap as pcap
import AcraNetwork.SimpleEthernet as SimpleEthernet
import argparse


parser = argparse.ArgumentParser(description='Benchmark the creation of pcap files containing packets')
parser.add_argument('--type',  required=True, type=str,choices=["udp","iena","inetx"],  help='The type of _payload, udp iena or inetx')
parser.add_argument('--ignoretime',required=False, action='store_true', default=False)
args = parser.parse_args()

# constants
PCAP_FNAME = "output_test.pcap"
PACKETS_TO_WRITE = 50000
PAYLOAD_SIZE = 1300 # size of the _payload in bytes
HEADER_SIZE = {'udp': 58, 'inetx':86,'iena':74}

# Write out a pcapfile with each inetx and iena packet generated
mypcap = pcap.Pcap(PCAP_FNAME, mode='w')
mypcap.write_global_header()
ethernet_packet = SimpleEthernet.Ethernet()
ethernet_packet.srcmac = 0x001122334455
ethernet_packet.dstmac = 0x554433221100
ethernet_packet.type = SimpleEthernet.Ethernet.TYPE_IP
ip_packet = SimpleEthernet.IP()
ip_packet.dstip = "235.0.0.2"
ip_packet.srcip = "127.0.0.1"
udp_packet = SimpleEthernet.UDP()
udp_packet.dstport = 4422


# Fixed _payload for both
payload = (struct.pack(">B",5) * PAYLOAD_SIZE)

if args.type == "inetx":
    # Create an inetx packet
    avionics_packet = inetx.iNetX()
    avionics_packet.inetxcontrol = inetx.iNetX.DEF_CONTROL_WORD
    avionics_packet.pif = 0
    avionics_packet.streamid = 0xdc
    avionics_packet.sequence = 0
    avionics_packet.payload = payload
else:
    # Create an iena packet
    avionics_packet = iena.IENA()
    avionics_packet.key = 0xdc
    avionics_packet.keystatus = 0
    avionics_packet.endfield = 0xbeef
    avionics_packet.sequence = 0
    avionics_packet.payload = payload
    avionics_packet.status = 0

packets_written = 0

start_time = time.time()
while packets_written < PACKETS_TO_WRITE:

    if args.type == "udp":
        udp_packet.srcport = 4999
        udp_packet.payload = payload
    else:
        if args.ignoretime:
            currenttime = 0
        else:
            currenttime = int(time.time())
        if args.type == "iena":
            avionics_packet.sequence = (avionics_packet.sequence + 1) % 65536
            udp_packet.srcport = 5000
        else:
            avionics_packet.sequence = (avionics_packet.sequence + 1) % 0x100000000
            udp_packet.srcport = 5001

        avionics_packet.setPacketTime(currenttime)
        udp_packet.payload = avionics_packet.pack()

    ip_packet.payload = udp_packet.pack()
    ethernet_packet.payload = ip_packet.pack()
    record = pcap.PcapRecord()
    if args.ignoretime:
        record.usec = 0
        record.sec = 0
    else:
        record.setCurrentTime()
    record.packet = ethernet_packet.pack()
    mypcap.write(record)

    packets_written += 1

mypcap.close()
end_time = time.time()
print("INFO: Wrote {} packets of type {} with _payload of {} bytes in {} seconds".format(PACKETS_TO_WRITE,args.type,PAYLOAD_SIZE,end_time-start_time))
print("INFO: Wrote {} bytes in {}".format((HEADER_SIZE[args.type]+PAYLOAD_SIZE)*PACKETS_TO_WRITE,end_time-start_time))
print("INFO: Wrote {} packets per second".format(PACKETS_TO_WRITE/(end_time-start_time)))
print("INFO: Wrote {:.2f} Mbps".format((HEADER_SIZE[args.type]+PAYLOAD_SIZE)*PACKETS_TO_WRITE*8/((end_time-start_time)*1024*1024)))


#-------------------------------------------------------------------------------
# Name:        module1
# Purpose:
#
# Author:      DCollins
#
# Created:     19/12/2013
# Copyright:   (c) DCollins 2013
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import colorama
from colorama import Fore, Back, Style
import os

def pos(y,x):
    return '\x1b[%d;%dH' % (y, x)

#pos = lambda y, x: '\x1b[%d;%dH' % (y, x)
def clear():
    if os.name == "nt":
        os.system('cls')
    else:
        os.system('reset')
    return
#clear = lambda: os.system('cls')


class ColouredOutput():
    """This class makes it easy to print out static tables on the console
    It's quite custom to iNetx packets"""
    # The starting locations on the screen
    MINY = 3
    MINX = 5
    # How long do we want the bar to be
    BAR_LEN = 14



    def __init__(self):
        """"""
        #----------------------------------
        # Setup the coloured output
        #----------------------------------
        colorama.init(autoreset=True)

        clear()

        self.y_location = ColouredOutput.MINY
        self.x_location = ColouredOutput.MINX
        # How the output will look
        self.output_format = "{:#10x} {:>15s} {:15d} {:>15s}"


        # This dict will contain a line number per streamid.
        self.YPosition = dict()
        self.lastyposition = self.y_location # Increment this on every streamid we find
        # We keep a count of the previous sequence number per streamid so that we can
        # identify dropped packets
        self.PreviousSeqNum = dict()

        # Print a nice vertical line with the temperature. the taller the line, the hotter it gets so chance
        # the colour of the line accordingly
        # Temperature colour gradient
        self.TempGradient = {1:colorama.Back.BLUE,2:colorama.Back.BLUE,3:colorama.Back.BLUE,4:colorama.Back.CYAN,5:colorama.Back.CYAN,
                            6:colorama.Back.CYAN,7:colorama.Back.YELLOW,8:colorama.Back.YELLOW,9:colorama.Back.MAGENTA,10:colorama.Back.MAGENTA,
                            11:colorama.Back.MAGENTA,12:colorama.Back.RED,13:colorama.Back.RED,14:colorama.Back.RED,15:colorama.Back.RED}

    def PrintHeader(self):
        """Print a header describing the columns"""
        outstring="{:>10s} {:>15s} {:>15s} {:>15s}".format("StreamID","Source IP","Sequence Num","TimeStamp")
        print(( pos(self.y_location, self.x_location) + colorama.Fore.BLACK+colorama.Back.RED + outstring))


    def PrintALine(self,string,sequencenum,streamid):
        """Print a line on the output one per streamid. The sequence number is incremented on each one"""
        # Figure out which line on the screen we should print at
        if streamid not in self.YPosition:
            self.lastyposition += 1
            self.YPosition[streamid] = self.lastyposition

        # Alternate the line colours to make it more readable
        if (sequencenum <= 100):
            # if we have recently reset keep this in red so it's obvious
            outputcolour = colorama.Fore.RED+colorama.Back.BLACK
        elif self.YPosition[streamid] % 2  == 0:
            outputcolour = colorama.Fore.CYAN+colorama.Back.BLACK
        else:
            outputcolour = colorama.Fore.GREEN+colorama.Back.BLACK


        # Print it to the correct position on the scree
        print(( pos(self.YPosition[streamid], self.x_location) + outputcolour + string))


    def PrintDroppedPacket(self,sequencenum,streamid,packet_count,pcapfilename):
        """Print out a warning if we have missed some sequence numbers"""
        # Check if we have dropped packets
        if streamid not in self.PreviousSeqNum:
            self.PreviousSeqNum[streamid] = sequencenum
        else:
            if self.PreviousSeqNum[streamid]+1 != sequencenum:
                # Dropped packet!
                print(( pos(self.YPosition[streamid], self.x_location+72) + colorama.Fore.MAGENTA+colorama.Back.CYAN+ "Dropped packet at count={} in {}".format(packet_count,pcapfilename)))

            self.PreviousSeqNum[streamid] = sequencenum

    def PrintTemperatureBar(self,temperature,offset):
        """This prints a verticl column with the temperature. Quite custom to one application"""
        bar_max_y = ColouredOutput.MINY + ColouredOutput.BAR_LEN - offset
        # So loop through the column and print accordingly
        for yposn in range(ColouredOutput.MINY, ColouredOutput.MINY + ColouredOutput.BAR_LEN):
            if yposn >= bar_max_y:
                print(( pos(yposn, 70) + self.TempGradient[offset] + "{: 3.1f}".format(temperature)))
            else:
                print(( pos(yposn, 70) + colorama.Back.BLACK + "     "))


    def PrintFileName(self,pcapfilename):
        """Print a header with the filename"""
        print(( pos(self.y_location-1, self.x_location) + colorama.Fore.BLACK+colorama.Back.RED + pcapfilename))

    def PrintExitInfo(self,string):
        print(( pos(self.y_location-1, self.x_location+30) + colorama.Fore.WHITE+colorama.Back.BLACK + string))


    def PrintMissingContinuity(self,streamid,mpegts):
        """Print out a warning if we have missed some sequence numbers"""
        # Check if we have dropped packets
        print(( pos(self.YPosition[streamid], self.x_location+72) + colorama.Fore.MAGENTA+colorama.Back.CYAN+ "Missed Continutiy"))




#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
===== 
MPEG OVER INETX 
===== 

This script listens to a udp port for inetx packets containing MPEG TS packets on reception it prints out the PIDs found
"""
__author__ = "Diarmuid Collins"
__copyright__ = "Copyright 2018"
__version__ = "0.0.1"
__maintainer__ = "Diarmuid Collins"
__email__ = "dcollins@curtisswright.com"
__status__ = "Production"


import socket,os,sys
sys.path.append("..")
import argparse
import AcraNetwork.iNetX as inetx
import AcraNetwork.MPEGTS as mpegts
import AcraNetwork.McastSocket as McastSocket
import struct

def align_payload(buf):
    byteindex = 0
    while len(buf) > byteindex:
            #print "idx={} len={}".format(byteindex,len(self._payload))
            (thisByte,) = struct.unpack_from('B',buf[byteindex])
            #logging.error("Byte = {:0X}".format(thisByte))
            if thisByte == 0x47:
                return buf[byteindex:]
            else:
                byteindex += 1
    return 0


def main():

    #----------------------------------
    # Setup the command line parser
    #----------------------------------
    parser = argparse.ArgumentParser(description='Analyse MPEG TS over inetx.')
    parser.add_argument('--inetx', type=int,  default=None, required=True,  help='Capture iNetX packets on this port')
    parser.add_argument('--multicast', type=str,  default="235.0.0.1", required=False,  help='The transmited multicast address')
    parser.add_argument('--localhost', type=str,  default='192.168.28.110', required=False,  help='The IP address of the local ethernet card')
    args = parser.parse_args()

    if args.inetx == None :
        parser.print_help()
        exit()


    PID_TEXT = {0x1fff : "Null", 0x0 : "Program Association Table", 0x100 : "VID106_Video" , 0x101 : "VID106_Audio", 0x1000 : "Program Map Table", 0x3E8 : "VID103_Video", 0x20 : "Unkn"}

    #------------------------------------------------------------
    # Setup a socket to recieve all traffic
    #------------------------------------------------------------
    #try:
    recv_socket = McastSocket.McastSocket(local_port=args.inetx, reuse=1)
    recv_socket.mcast_add(args.multicast, args.localhost)
    recv_socket.settimeout(10)
    #except:
    #    print "Can't bind to socket {}".format(args.inetx)
    #    print sys.exc_info()[0]
    #    exit()


    aligned_payload = ""

    while True:

        # Capture some data from the socket
        try:
            data, addr = recv_socket.recvfrom(2048) # buffer size is 1500 bytes
        except socket.timeout:
            print("timeout on socket")
            exit()

        # we are expecting a iNetX packets
        inetxpacket = inetx.iNetX()
        try:
            inetxpacket.unpack(data)
        except:
            pass

        # if we are in snarfer mode then align the _payload
        aligned_payload = align_payload(aligned_payload+inetxpacket.payload)
        if len(aligned_payload) >= 1316:

            # take the aligned _payload and convert it into MPEG TS blocks
            # Then print out the blocks
            mpegtspackets = mpegts.MPEGTS()
            mpegtspackets.unpack(aligned_payload[:1316])
            for mpegblock in mpegtspackets.blocks:
                print("PID = {}".format(PID_TEXT[mpegblock.pid]))

            aligned_payload = aligned_payload[1316:]


if __name__ == '__main__':
    main()



#!/usr/bin/env python

import socket
import struct
import time
import sys
import argparse
import subprocess
import os


VERSION = "0.2.0"



def create_parser():
    # Argument parser
    parser = argparse.ArgumentParser(description='Validate inetx sequence numbers quickly')
    # Common
    parser.add_argument('--port',  type=int, required=False, default=8010,
                        help='UDP port to listen to')
    parser.add_argument('--multicastaddr',  type=str, required=False, default="235.0.0.1",
                        help='multicast address to listen to')
    parser.add_argument('--verbose', required=False, action='store_true', default=False,    help="verbose mode")
    parser.add_argument('--version', action='version', version='%(prog)s {}'.format(VERSION))

    return parser

def open_socket(args):
    server_address = ('', args.port)

    # Create the socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    sock.settimeout(10)

    # Bind to the server address
    sock.bind(server_address)
    # Tell the operating system to add the socket to the multicast group
    # on all interfaces.
    group = socket.inet_aton(args.multicastaddr)
    mreq = struct.pack('4sL', group, socket.INADDR_ANY)
    sock.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP, mreq)
    if os.name != 'nt':
        # Hack to get Limux to deliver mcast packets to python
        subprocess.call(["/sbin/route", "add", args.multicastaddr, "enp2s0"])

    return sock

def main(args):
    sock = open_socket(args)

    # Guts of the program
    sequence_count = {}
    sequence_error_count = {}
    pkt_count = 0
    err_cnt = 0
    seq_rollover = pow(2,32)


    start_time = time.time()
    while True:
        data, address = sock.recvfrom(2048)
        (streamid, sequence) = struct.unpack_from(">II", data, 4)
        if streamid in sequence_count:
            if (sequence_count[streamid] + 1) % seq_rollover != sequence:
                print ("ERROR: Seq error on {:#0X}. Prev={} Current={} Lost={} Time={}".format(
                    streamid, sequence_count[streamid], sequence, sequence-sequence_count[streamid], time.strftime("%X %x")))
                err_cnt += 1
                sequence_error_count[streamid] += 1
        sequence_count[streamid] = sequence
        sequence_error_count[streamid] = 0
        pkt_count += 1

        if pkt_count % 10000 == 0:
            cur_time = time.time()
            print ("INFO: Received {} packets in {:4.0f} seconds. Error Count={}".format(pkt_count, cur_time - start_time, err_cnt))
            for stream, seq in sequence_count.iteritems():
                print ("INFO: StreamID={:#0X} CurSeq={}".format(stream, seq))


            sys.stdout.flush()

if __name__ == '__main__':
    parser = create_parser()
    args = parser.parse_args()
    main(args)
    sys.exit(0)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
===== 
Benchmark PCAP Creaion
===== 

Benchmark the transmission of multicast packets
"""
__author__ = "Diarmuid Collins"
__copyright__ = "Copyright 2018"
__version__ = "0.0.1"
__maintainer__ = "Diarmuid Collins"
__email__ = "dcollins@curtisswright.com"
__status__ = "Production"


import sys
sys.path.append("..")
import time
import struct
import AcraNetwork.IENA as iena
import AcraNetwork.iNetX as inetx
import AcraNetwork.McastSocket as mcast
import AcraNetwork.Pcap as pcap
import AcraNetwork.SimpleEthernet as SimpleEthernet
import argparse

parser = argparse.ArgumentParser(description='Benchmark the transmission of multicast packets')
parser.add_argument('--type',  required=True, type=str,choices=["udp","iena","inetx"],  help='The type of _payload, udp iena or inetx')
parser.add_argument('--ignoretime',required=False, action='store_true', default=False)
args = parser.parse_args()

# constants
PACKETS_TO_SEND = 50000
PAYLOAD_SIZE = 1300 # size of the _payload in bytes
HEADER_SIZE = {'udp' : 58 , 'inetx' :86 ,'iena':74}
UDP_IP = "235.0.0.1"
UDP_PORT = 8888

# Fixed _payload for both
payload = (struct.pack(">B",5) * PAYLOAD_SIZE)

if args.type == "inetx":
    # Create an inetx packet
    avionics_packet = inetx.iNetX()
    avionics_packet.inetxcontrol = inetx.iNetX.DEF_CONTROL_WORD
    avionics_packet.pif = 0
    avionics_packet.streamid = 0xdc
    avionics_packet.sequence = 0
    avionics_packet.payload = payload
elif args.type == "iena":
    # Create an iena packet
    avionics_packet = iena.IENA()
    avionics_packet.key = 0xdc
    avionics_packet.keystatus = 0
    avionics_packet.endfield = 0xbeef
    avionics_packet.sequence = 0
    avionics_packet.payload = payload
    avionics_packet.status = 0

mcastsocket = mcast.McastSocket(2048)
mcastsocket.mcast_add(UDP_IP)

packets_sent = 0

start_time = time.time()
while packets_sent < PACKETS_TO_SEND:
    if args.type == "udp":
        packet_payload = payload
    else:
        if args.ignoretime:
            currenttime = 0
        else:
            currenttime = int(time.time())
        if args.type == "iena":
            avionics_packet.sequence = (avionics_packet.sequence +1) % 65536
        else:
            avionics_packet.sequence = (avionics_packet.sequence +1) % 0x100000000

        avionics_packet.setPacketTime(currenttime)
        packet_payload = avionics_packet.pack()

    mcastsocket.sendto(packet_payload, (UDP_IP, UDP_PORT))
    packets_sent += 1

end_time = time.time()
print("INFO: Sent {} packets of type {} with _payload of {} bytes in {} seconds".format(PACKETS_TO_SEND,args.type,PAYLOAD_SIZE,end_time-start_time))
print("INFO: Sent {} bytes in {}".format((HEADER_SIZE[args.type]+PAYLOAD_SIZE)*PACKETS_TO_SEND,end_time-start_time))
print("INFO: Sent {} packets per second".format(PACKETS_TO_SEND/(end_time-start_time)))
print("INFO: Sent {:.2f} Mbps".format((HEADER_SIZE[args.type]+PAYLOAD_SIZE)*PACKETS_TO_SEND*8/((end_time-start_time)*1024*1024)))


#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
===== 
Benchmark Multicast UDP Reception
===== 

Benchmark the reception of multicast packets
"""
__author__ = "Diarmuid Collins"
__copyright__ = "Copyright 2018"
__version__ = "0.0.1"
__maintainer__ = "Diarmuid Collins"
__email__ = "dcollins@curtisswright.com"
__status__ = "Production"



import sys
sys.path.append("..")


import time
import struct
import AcraNetwork.IENA as iena
import AcraNetwork.iNetX as inetx
import AcraNetwork.McastSocket as mcast
import AcraNetwork.SimpleEthernet as SimpleEthernet
import argparse
from collections import deque


def main():

    # Setup some constants
    PACKETS_TO_RECEIVE = 1000

    #----------------------------------
    # Setup the command line parser
    #----------------------------------
    parser = argparse.ArgumentParser(description='Live Analysis of BCU transmitted packets.')
    parser.add_argument('--inetx', type=int,  default=None, required=False,  help='Receiving iNetX packets on this UDP port')
    parser.add_argument('--iena', type=int,  default=None, required=False,  help='Receiving IENA packets on this UDP port')
    parser.add_argument('--address', type=str,  default="235.0.0.1", required=False,  help='Destination multicast address')
    args = parser.parse_args()

    if args.inetx != None:
        udp_port = args.inetx
        is_inetx = True
    elif args.iena != None:
        udp_port = args.iena
        is_inetx = False
    else:
        parser.print_help()
        exit()

    #------------------------------------------------------------
    # Setup a socket to receive all traffic
    #------------------------------------------------------------
    try:
        recv_socket = mcast.McastSocket(local_port=udp_port, reuse=1)
        recv_socket.mcast_add(args.address, '0.0.0.0')
        recv_socket.settimeout(3)
    except:
        print("Can't bind to socket {}".format(udp_port))
        exit()

    start_time = time.time()
    packet_count = 1
    drop_count = 0
    streams = dict()
    data_len = deque()


    while packet_count < PACKETS_TO_RECEIVE:

        # Capture some data from the socket
        try:
            data, addr = recv_socket.recvfrom(2048) # buffer size is 1500 bytes
        except:
            print("timeout on socket")
            exit()

        (udpsrcport,srcipaddr) = addr

        # keep a track of the sizes of the last 20 packets received so that we
        # can calculate the bandwitdh
        # The data returned in the UDP _payload only so add 42 bytes for the rest of the
        # ethernet header just so the calculated number agress with Wireshark
        data_len.append(len(data)+42)
        if len(data_len) > 20:
            data_len.popleft()

        # Create a new packet depending on the command line arguments
        # and unpack the received data into these objects
        if is_inetx:
            avionics_packet = inetx.iNetX()
            avionics_packet.unpack(data)
            try:
                avionics_packet.unpack(data)
            except ValueError:
                # This isn't an inetx packet
                packet_count += 1
                continue
        else:
            avionics_packet = iena.IENA()
            try:
                avionics_packet.unpack(data)
            except ValueError:
                # Not a valid IENA packet
                packet_count += 1
                continue

        if is_inetx:
            if avionics_packet.streamid in streams:
                if avionics_packet.sequence != ((streams[avionics_packet.streamid] +1) % 0x100000000):
                    drop_count += 1
            streams[avionics_packet.streamid] = avionics_packet.sequence

        else:
            if avionics_packet.key in streams:
                if avionics_packet.sequence != ((streams[avionics_packet.key] +1) % 65536):
                    drop_count += 1
            streams[avionics_packet.key] = avionics_packet.sequence



        packet_count += 1

    end_time = time.time()
    average_len = sum(data_len)/len(data_len)
    print("INFO: Recevied {} packets in {} seconds with {} dropped packets".format(packet_count,end_time-start_time,drop_count))
    print("INFO: Recevied {:.3f} Mbps".format((average_len*packet_count*8)/((end_time-start_time)*1024*1024)))

if __name__ == '__main__':
    main()




import sys
sys.path.append("..")

import AcraNetwork.Pcap as pcap
import csv
import AcraNetwork.iNetX as inetx


# Modify this
pcapf = "../test/inetx_test.pcap"
streamid = 0xca
csv_output = "ts.csv"

csvfile = open(csv_output, 'w')
csvwriter = csv.writer(csvfile)
csvwriter.writerow(["Packet Number", "Seconds", "Nanoseconds"])

pcapfile = pcap.Pcap(pcapf, mode="r")
for idx, record in enumerate(pcapfile):
    inetx_pkt = inetx.iNetX()
    try:
        inetx_pkt.unpack(record.payload[0x2a:])
    except:
        pass
    else:
        if inetx_pkt.streamid == streamid:
            csvwriter.writerow([idx+1, inetx_pkt.ptptimeseconds, inetx_pkt.ptptimenanoseconds])